@@ -23,6 +23,7 @@
 
 #include <bson.h>
 
+#include "mongoc-macros.h"
 #include "mongoc-apm.h"
 #include "mongoc-client.h"
 #include "mongoc-config.h"
@@ -38,34 +39,34 @@ BSON_BEGIN_DECLS
 typedef struct _mongoc_client_pool_t mongoc_client_pool_t;
 
 
-BSON_EXPORT (mongoc_client_pool_t *)
+MONGOC_EXPORT (mongoc_client_pool_t *)
 mongoc_client_pool_new (const mongoc_uri_t *uri);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_client_pool_destroy (mongoc_client_pool_t *pool);
-BSON_EXPORT (mongoc_client_t *)
+MONGOC_EXPORT (mongoc_client_t *)
 mongoc_client_pool_pop (mongoc_client_pool_t *pool);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_client_pool_push (mongoc_client_pool_t *pool, mongoc_client_t *client);
-BSON_EXPORT (mongoc_client_t *)
+MONGOC_EXPORT (mongoc_client_t *)
 mongoc_client_pool_try_pop (mongoc_client_pool_t *pool);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_client_pool_max_size (mongoc_client_pool_t *pool,
                              uint32_t max_pool_size);
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_client_pool_min_size (mongoc_client_pool_t *pool,
-                             uint32_t min_pool_size);
+                             uint32_t min_pool_size) BSON_GNUC_DEPRECATED;
 #ifdef MONGOC_ENABLE_SSL
-BSON_EXPORT (void)
+MONGOC_EXPORT (void)
 mongoc_client_pool_set_ssl_opts (mongoc_client_pool_t *pool,
                                  const mongoc_ssl_opt_t *opts);
 #endif
-BSON_EXPORT (bool)
+MONGOC_EXPORT (bool)
 mongoc_client_pool_set_apm_callbacks (mongoc_client_pool_t *pool,
                                       mongoc_apm_callbacks_t *callbacks,
                                       void *context);
-BSON_EXPORT (bool)
+MONGOC_EXPORT (bool)
 mongoc_client_pool_set_error_api (mongoc_client_pool_t *pool, int32_t version);
-BSON_EXPORT (bool)
+MONGOC_EXPORT (bool)
 mongoc_client_pool_set_appname (mongoc_client_pool_t *pool,
                                 const char *appname);
 BSON_END_DECLS
