@@ -33,7 +33,6 @@
 #endif
 #endif
 #include "mongoc-thread-private.h"
-#include "mongoc-trace-private.h"
 
 
 #ifndef MONGOC_NO_AUTOMATIC_GLOBALS
@@ -45,10 +44,9 @@
 
 #ifdef MONGOC_ENABLE_SASL_CYRUS
 #include <sasl/sasl.h>
-#include "mongoc-sasl-private.h"
 
 static void *
-mongoc_sasl_mutex_alloc (void)
+mongoc_cyrus_mutex_alloc (void)
 {
    mongoc_mutex_t *mutex;
 
@@ -60,7 +58,7 @@ mongoc_sasl_mutex_alloc (void)
 
 
 static int
-mongoc_sasl_mutex_lock (void *mutex)
+mongoc_cyrus_mutex_lock (void *mutex)
 {
    mongoc_mutex_lock ((mongoc_mutex_t *) mutex);
 
@@ -69,7 +67,7 @@ mongoc_sasl_mutex_lock (void *mutex)
 
 
 static int
-mongoc_sasl_mutex_unlock (void *mutex)
+mongoc_cyrus_mutex_unlock (void *mutex)
 {
    mongoc_mutex_unlock ((mongoc_mutex_t *) mutex);
 
@@ -78,7 +76,7 @@ mongoc_sasl_mutex_unlock (void *mutex)
 
 
 static void
-mongoc_sasl_mutex_free (void *mutex)
+mongoc_cyrus_mutex_free (void *mutex)
 {
    mongoc_mutex_destroy ((mongoc_mutex_t *) mutex);
    bson_free (mutex);
@@ -91,9 +89,6 @@ static MONGOC_ONCE_FUN (_mongoc_do_init)
 {
 #ifdef MONGOC_ENABLE_SASL_CYRUS
    int status;
-   sasl_callback_t callbacks[] = {
-      {SASL_CB_LOG, SASL_CALLBACK_FN (_mongoc_sasl_log), NULL},
-      {SASL_CB_LIST_END}};
 #endif
 #ifdef MONGOC_ENABLE_SSL_OPENSSL
    _mongoc_openssl_init ();
@@ -108,12 +103,12 @@ static MONGOC_ONCE_FUN (_mongoc_do_init)
 #ifdef MONGOC_ENABLE_SASL_CYRUS
    /* The following functions should not use tracing, as they may be invoked
     * before mongoc_log_set_handler() can complete. */
-   sasl_set_mutex (mongoc_sasl_mutex_alloc,
-                   mongoc_sasl_mutex_lock,
-                   mongoc_sasl_mutex_unlock,
-                   mongoc_sasl_mutex_free);
+   sasl_set_mutex (mongoc_cyrus_mutex_alloc,
+                   mongoc_cyrus_mutex_lock,
+                   mongoc_cyrus_mutex_unlock,
+                   mongoc_cyrus_mutex_free);
 
-   status = sasl_client_init (callbacks);
+   status = sasl_client_init (NULL);
    BSON_ASSERT (status == SASL_OK);
 #endif
 
