@@ -22,6 +22,29 @@
 #include "mongoc-util-private.h"
 #include "mongoc-read-prefs-private.h"
 #include "mongoc-set-private.h"
+#include "mongoc-client-private.h"
+#include "mongoc-thread-private.h"
+
+
+static bool
+_is_data_node (mongoc_server_description_t *sd)
+{
+   switch (sd->type) {
+   case MONGOC_SERVER_MONGOS:
+   case MONGOC_SERVER_STANDALONE:
+   case MONGOC_SERVER_RS_SECONDARY:
+   case MONGOC_SERVER_RS_PRIMARY:
+      return true;
+   case MONGOC_SERVER_RS_OTHER:
+   case MONGOC_SERVER_RS_ARBITER:
+   case MONGOC_SERVER_UNKNOWN:
+   case MONGOC_SERVER_POSSIBLE_PRIMARY:
+   case MONGOC_SERVER_RS_GHOST:
+   case MONGOC_SERVER_DESCRIPTION_TYPES:
+   default:
+      return false;
+   }
+}
 
 
 static void
@@ -47,30 +70,26 @@ _mongoc_topology_server_dtor (void *server_, void *ctx_)
  */
 void
 mongoc_topology_description_init (mongoc_topology_description_t *description,
-                                  mongoc_topology_description_type_t type,
                                   int64_t heartbeat_msec)
 {
    ENTRY;
 
    BSON_ASSERT (description);
-   BSON_ASSERT (type == MONGOC_TOPOLOGY_UNKNOWN ||
-                type == MONGOC_TOPOLOGY_SINGLE ||
-                type == MONGOC_TOPOLOGY_RS_NO_PRIMARY);
 
    memset (description, 0, sizeof (*description));
 
    bson_oid_init (&description->topology_id, NULL);
    description->opened = false;
-   description->type = type;
+   description->type = MONGOC_TOPOLOGY_UNKNOWN;
    description->heartbeat_msec = heartbeat_msec;
    description->servers =
       mongoc_set_new (8, _mongoc_topology_server_dtor, NULL);
    description->set_name = NULL;
    description->max_set_version = MONGOC_NO_SET_VERSION;
-   description->compatible = true;
-   description->compatibility_error = NULL;
    description->stale = true;
    description->rand_seed = (unsigned int) bson_get_monotonic_time ();
+   bson_init (&description->cluster_time);
+   description->session_timeout_minutes = MONGOC_NO_SESSIONS;
 
    EXIT;
 }
@@ -123,8 +142,9 @@ _mongoc_topology_description_copy_to (const mongoc_topology_description_t *src,
 
    dst->set_name = bson_strdup (src->set_name);
    dst->max_set_version = src->max_set_version;
-   dst->compatible = src->compatible;
-   dst->compatibility_error = bson_strdup (src->compatibility_error);
+   memcpy (&dst->compatibility_error,
+           &src->compatibility_error,
+           sizeof (bson_error_t));
    dst->max_server_id = src->max_server_id;
    dst->stale = src->stale;
    memcpy (&dst->apm_callbacks,
@@ -133,6 +153,10 @@ _mongoc_topology_description_copy_to (const mongoc_topology_description_t *src,
 
    dst->apm_context = src->apm_context;
 
+   bson_copy_to (&src->cluster_time, &dst->cluster_time);
+
+   dst->session_timeout_minutes = src->session_timeout_minutes;
+
    EXIT;
 }
 
@@ -158,15 +182,15 @@ mongoc_topology_description_destroy (mongoc_topology_description_t *description)
 
    BSON_ASSERT (description);
 
-   mongoc_set_destroy (description->servers);
+   if (description->servers) {
+      mongoc_set_destroy (description->servers);
+   }
 
    if (description->set_name) {
       bson_free (description->set_name);
    }
 
-   if (description->compatibility_error) {
-      bson_free (description->compatibility_error);
-   }
+   bson_destroy (&description->cluster_time);
 
    EXIT;
 }
@@ -713,6 +737,30 @@ DONE:
    return;
 }
 
+/*
+ *--------------------------------------------------------------------------
+ *
+ * mongoc_topology_description_has_data_node --
+ *
+ *      Internal method: are any servers not Arbiter, Ghost, or Unknown?
+ *
+ *--------------------------------------------------------------------------
+ */
+bool
+mongoc_topology_description_has_data_node (mongoc_topology_description_t *td)
+{
+   int i;
+   mongoc_server_description_t *sd;
+
+   for (i = 0; i < (int) td->servers->items_len; i++) {
+      sd = (mongoc_server_description_t *) mongoc_set_get_item (td->servers, i);
+      if (_is_data_node (sd)) {
+         return true;
+      }
+   }
+
+   return false;
+}
 
 /*
  *-------------------------------------------------------------------------
@@ -749,12 +797,6 @@ mongoc_topology_description_select (mongoc_topology_description_t *topology,
 
    ENTRY;
 
-   if (!topology->compatible) {
-      /* TODO: check this in mongoc_topology_compatible (), CDRIVER-689 */
-      TRACE ("%s", "Incompatible topology");
-      RETURN (NULL);
-   }
-
    if (topology->type == MONGOC_TOPOLOGY_SINGLE) {
       sd = (mongoc_server_description_t *) mongoc_set_get_item (
          topology->servers, 0);
@@ -773,7 +815,7 @@ mongoc_topology_description_select (mongoc_topology_description_t *topology,
    mongoc_topology_description_suitable_servers (
       &suitable_servers, optype, topology, read_pref, local_threshold_ms);
    if (suitable_servers.len != 0) {
-      rand_n = MONGOC_RAND_R (&topology->rand_seed);
+      rand_n = _mongoc_rand_simple (&topology->rand_seed);
       sd = _mongoc_array_index (&suitable_servers,
                                 mongoc_server_description_t *,
                                 rand_n % suitable_servers.len);
@@ -809,7 +851,7 @@ mongoc_topology_description_select (mongoc_topology_description_t *topology,
  *       A mongoc_server_description_t *, or NULL.
  *
  * Side effects:
- *      Fills out optional @error if server not found.
+ *       Fills out optional @error if server not found.
  *
  *--------------------------------------------------------------------------
  */
@@ -1142,6 +1184,69 @@ mongoc_topology_description_add_server (mongoc_topology_description_t *topology,
 }
 
 
+/*
+ *--------------------------------------------------------------------------
+ *
+ * mongoc_topology_description_update_cluster_time --
+ *
+ *  Drivers Session Spec: Drivers MUST examine responses to server commands to
+ *  see if they contain a top level field named $clusterTime formatted as
+ *  follows:
+ *
+ *  {
+ *      ...
+ *      $clusterTime : {
+ *          clusterTime : <BsonTimestamp>,
+ *          signature : {
+ *              hash : <BsonBinaryData>,
+ *              keyId : <BsonInt64>
+ *          }
+ *      },
+ *      ...
+ *  }
+ *
+ *  Whenever a driver receives a clusterTime from a server it MUST compare it
+ *  to the current highest seen clusterTime for the cluster. If the new
+ *  clusterTime is higher than the highest seen clusterTime it MUST become
+ *  the new highest seen clusterTime. Two clusterTimes are compared using
+ *  only the BsonTimestamp value of the clusterTime embedded field (be sure to
+ *  include both the timestamp and the increment of the BsonTimestamp in the
+ *  comparison). The signature field does not participate in the comparison.
+ *
+ *--------------------------------------------------------------------------
+ */
+
+void
+mongoc_topology_description_update_cluster_time (
+   mongoc_topology_description_t *td, const bson_t *reply)
+{
+   bson_iter_t iter;
+   bson_iter_t child;
+   const uint8_t *data;
+   uint32_t size;
+   bson_t cluster_time;
+
+   if (!reply || !bson_iter_init_find (&iter, reply, "$clusterTime")) {
+      return;
+   }
+
+   if (!BSON_ITER_HOLDS_DOCUMENT (&iter) ||
+       !bson_iter_recurse (&iter, &child)) {
+      MONGOC_ERROR ("Can't parse $clusterTime");
+      return;
+   }
+
+   bson_iter_document (&iter, &size, &data);
+   bson_init_static (&cluster_time, data, (size_t) size);
+
+   if (bson_empty (&td->cluster_time) ||
+       _mongoc_cluster_time_greater (&cluster_time, &td->cluster_time)) {
+      bson_destroy (&td->cluster_time);
+      bson_copy_to (&cluster_time, &td->cluster_time);
+   }
+}
+
+
 static void
 _mongoc_topology_description_add_new_servers (
    mongoc_topology_description_t *topology, mongoc_server_description_t *server)
@@ -1400,16 +1505,16 @@ _mongoc_topology_description_update_rs_without_primary (
    /* Add new servers that this replica set member knows about */
    _mongoc_topology_description_add_new_servers (topology, server);
 
-   if (!_mongoc_topology_description_matches_me (server)) {
-      _mongoc_topology_description_remove_server (topology, server);
-      return;
-   }
-
    /* If this server thinks there is a primary, label it POSSIBLE_PRIMARY */
    if (server->current_primary) {
       _mongoc_topology_description_label_unknown_member (
          topology, server->current_primary, MONGOC_SERVER_POSSIBLE_PRIMARY);
    }
+
+   if (!_mongoc_topology_description_matches_me (server)) {
+      _mongoc_topology_description_remove_server (topology, server);
+      return;
+   }
 }
 
 /*
@@ -1602,7 +1707,7 @@ transition_t gSDAMTransitionTable
          NULL, /* MONGOC_TOPOLOGY_SHARDED */
          NULL, /* MONGOC_TOPOLOGY_RS_NO_PRIMARY */
          _mongoc_topology_description_check_if_has_primary /* MONGOC_TOPOLOGY_RS_WITH_PRIMARY
-                                                              */
+                                                            */
       },
       {/* STANDALONE */
        _mongoc_topology_description_update_unknown_with_standalone,
@@ -1687,6 +1792,106 @@ _mongoc_topology_description_type (mongoc_topology_description_t *topology)
 }
 #endif
 
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * _mongoc_topology_description_update_session_timeout --
+ *
+ *      Fill out td.session_timeout_minutes.
+ *
+ *      Server Discovery and Monitoring Spec: "set logicalSessionTimeoutMinutes
+ *      to the smallest logicalSessionTimeoutMinutes value among all
+ *      ServerDescriptions of known ServerType. If any ServerDescription of
+ *      known ServerType has a null logicalSessionTimeoutMinutes, then
+ *      logicalSessionTimeoutMinutes MUST be set to null."
+ *
+ * --------------------------------------------------------------------------
+ */
+
+static void
+_mongoc_topology_description_update_session_timeout (
+   mongoc_topology_description_t *td)
+{
+   mongoc_set_t *set;
+   size_t i;
+   mongoc_server_description_t *sd;
+
+   set = td->servers;
+
+   td->session_timeout_minutes = MONGOC_NO_SESSIONS;
+
+   for (i = 0; i < set->items_len; i++) {
+      sd = (mongoc_server_description_t *) mongoc_set_get_item (set, (int) i);
+      if (!_is_data_node (sd)) {
+         continue;
+      }
+
+      if (sd->session_timeout_minutes == MONGOC_NO_SESSIONS) {
+         td->session_timeout_minutes = MONGOC_NO_SESSIONS;
+         return;
+      } else if (td->session_timeout_minutes == MONGOC_NO_SESSIONS) {
+         td->session_timeout_minutes = sd->session_timeout_minutes;
+      } else if (td->session_timeout_minutes > sd->session_timeout_minutes) {
+         td->session_timeout_minutes = sd->session_timeout_minutes;
+      }
+   }
+}
+
+/*
+ *--------------------------------------------------------------------------
+ *
+ * _mongoc_topology_description_check_compatible --
+ *
+ *      Fill out td.compatibility_error if any server's wire versions do
+ *      not overlap with ours. Otherwise clear td.compatibility_error.
+ *
+ *      If any server is incompatible, the topology as a whole is considered
+ *      incompatible.
+ *
+ *--------------------------------------------------------------------------
+ */
+
+static void
+_mongoc_topology_description_check_compatible (
+   mongoc_topology_description_t *td)
+{
+   size_t i;
+   mongoc_server_description_t *sd;
+
+   memset (&td->compatibility_error, 0, sizeof (bson_error_t));
+
+   for (i = 0; i < td->servers->items_len; i++) {
+      sd = (mongoc_server_description_t *) mongoc_set_get_item (td->servers,
+                                                                (int) i);
+      if (sd->type == MONGOC_SERVER_UNKNOWN ||
+          sd->type == MONGOC_SERVER_POSSIBLE_PRIMARY) {
+         continue;
+      }
+
+      if (sd->min_wire_version > WIRE_VERSION_MAX) {
+         bson_set_error (
+            &td->compatibility_error,
+            MONGOC_ERROR_PROTOCOL,
+            MONGOC_ERROR_PROTOCOL_BAD_WIRE_VERSION,
+            "Server at %s requires wire version %d,"
+            " but this version of libmongoc only supports up to %d",
+            sd->host.host_and_port,
+            sd->min_wire_version,
+            WIRE_VERSION_MAX);
+      } else if (sd->max_wire_version < WIRE_VERSION_MIN) {
+         bson_set_error (
+            &td->compatibility_error,
+            MONGOC_ERROR_PROTOCOL,
+            MONGOC_ERROR_PROTOCOL_BAD_WIRE_VERSION,
+            "Server at %s reports wire version %d, but this"
+            " version of libmongoc requires at least 2 (MongoDB 2.6)",
+            sd->host.host_and_port,
+            sd->max_wire_version);
+      }
+   }
+}
+
 /*
  *--------------------------------------------------------------------------
  *
@@ -1735,6 +1940,8 @@ mongoc_topology_description_handle_ismaster (
    mongoc_server_description_handle_ismaster (
       sd, ismaster_response, rtt_msec, error);
 
+   mongoc_topology_description_update_cluster_time (topology,
+                                                    ismaster_response);
    _mongoc_topology_description_monitor_server_changed (topology, prev_sd, sd);
 
    if (gSDAMTransitionTable[sd->type][topology->type]) {
@@ -1748,6 +1955,12 @@ mongoc_topology_description_handle_ismaster (
              mongoc_server_description_type (sd));
    }
 
+   _mongoc_topology_description_update_session_timeout (topology);
+
+   /* Don't bother checking wire version compatibility if we already errored */
+   if (ismaster_response && (!error || !error->code)) {
+      _mongoc_topology_description_check_compatible (topology);
+   }
    _mongoc_topology_description_monitor_changed (prev_td, topology);
 
    if (prev_td) {
@@ -1778,6 +1991,12 @@ bool
 mongoc_topology_description_has_readable_server (
    mongoc_topology_description_t *td, const mongoc_read_prefs_t *prefs)
 {
+   bson_error_t error;
+
+   if (!mongoc_topology_compatible (td, NULL, &error)) {
+      return false;
+   }
+
    /* local threshold argument doesn't matter */
    return mongoc_topology_description_select (td, MONGOC_SS_READ, prefs, 0) !=
           NULL;
@@ -1801,6 +2020,12 @@ bool
 mongoc_topology_description_has_writable_server (
    mongoc_topology_description_t *td)
 {
+   bson_error_t error;
+
+   if (!mongoc_topology_compatible (td, NULL, &error)) {
+      return false;
+   }
+
    return mongoc_topology_description_select (td, MONGOC_SS_WRITE, NULL, 0) !=
           NULL;
 }
@@ -1839,11 +2064,12 @@ mongoc_topology_description_type (const mongoc_topology_description_t *td)
    case MONGOC_TOPOLOGY_DESCRIPTION_TYPES:
    default:
       fprintf (stderr, "ERROR: Unknown topology type %d\n", td->type);
-      assert (0);
+      BSON_ASSERT (0);
    }
 
    return NULL;
 }
+
 /*
  *--------------------------------------------------------------------------
  *
