@@ -27,13 +27,18 @@
 BSON_BEGIN_DECLS
 
 
-void
-mongoc_uri_lowercase_hostname (const char *src, char *buf /* OUT */, int len);
 bool
-mongoc_uri_append_host (mongoc_uri_t *uri, const char *host, uint16_t port);
+mongoc_uri_append_host (mongoc_uri_t *uri,
+                        const char *host,
+                        uint16_t port,
+                        bson_error_t *error);
 bool
-mongoc_uri_parse_host (mongoc_uri_t *uri, const char *str);
-
+mongoc_uri_parse_host (mongoc_uri_t *uri, const char *str, bool downcase);
+bool
+mongoc_uri_parse_options (mongoc_uri_t *uri,
+                          const char *str,
+                          bool from_dns,
+                          bson_error_t *error);
 int32_t
 mongoc_uri_get_local_threshold_option (const mongoc_uri_t *uri);
 
