@@ -17,20 +17,11 @@
 
 #include "mongoc-error.h"
 #include "mongoc-log.h"
+#include "mongoc-util-private.h"
 #include "mongoc-write-concern.h"
 #include "mongoc-write-concern-private.h"
 
 
-static BSON_INLINE bool
-_mongoc_write_concern_warn_frozen (mongoc_write_concern_t *write_concern)
-{
-   if (write_concern->frozen) {
-      MONGOC_WARNING ("Cannot modify a frozen write-concern.");
-   }
-
-   return write_concern->frozen;
-}
-
 static void
 _mongoc_write_concern_freeze (mongoc_write_concern_t *write_concern);
 
@@ -55,6 +46,8 @@ mongoc_write_concern_new (void)
    write_concern->journal = MONGOC_WRITE_CONCERN_JOURNAL_DEFAULT;
    write_concern->is_default = true;
 
+   bson_init (&write_concern->compiled);
+
    return write_concern;
 }
 
@@ -89,11 +82,7 @@ void
 mongoc_write_concern_destroy (mongoc_write_concern_t *write_concern)
 {
    if (write_concern) {
-      if (write_concern->compiled.len) {
-         bson_destroy (&write_concern->compiled);
-         bson_destroy (&write_concern->compiled_gle);
-      }
-
+      bson_destroy (&write_concern->compiled);
       bson_free (write_concern->wtag);
       bson_free (write_concern);
    }
@@ -122,10 +111,9 @@ mongoc_write_concern_set_fsync (mongoc_write_concern_t *write_concern,
 {
    BSON_ASSERT (write_concern);
 
-   if (!_mongoc_write_concern_warn_frozen (write_concern)) {
-      write_concern->fsync_ = !!fsync_;
-      write_concern->is_default = false;
-   }
+   write_concern->fsync_ = !!fsync_;
+   write_concern->is_default = false;
+   write_concern->frozen = false;
 }
 
 
@@ -160,10 +148,9 @@ mongoc_write_concern_set_journal (mongoc_write_concern_t *write_concern,
 {
    BSON_ASSERT (write_concern);
 
-   if (!_mongoc_write_concern_warn_frozen (write_concern)) {
-      write_concern->journal = !!journal;
-      write_concern->is_default = false;
-   }
+   write_concern->journal = !!journal;
+   write_concern->is_default = false;
+   write_concern->frozen = false;
 }
 
 
@@ -192,12 +179,11 @@ mongoc_write_concern_set_w (mongoc_write_concern_t *write_concern, int32_t w)
    BSON_ASSERT (write_concern);
    BSON_ASSERT (w >= -3);
 
-   if (!_mongoc_write_concern_warn_frozen (write_concern)) {
-      write_concern->w = w;
-      if (w != MONGOC_WRITE_CONCERN_W_DEFAULT) {
-         write_concern->is_default = false;
-      }
+   write_concern->w = w;
+   if (w != MONGOC_WRITE_CONCERN_W_DEFAULT) {
+      write_concern->is_default = false;
    }
+   write_concern->frozen = false;
 }
 
 
@@ -230,10 +216,9 @@ mongoc_write_concern_set_wtimeout (mongoc_write_concern_t *write_concern,
       return;
    }
 
-   if (!_mongoc_write_concern_warn_frozen (write_concern)) {
-      write_concern->wtimeout = wtimeout_msec;
-      write_concern->is_default = false;
-   }
+   write_concern->wtimeout = wtimeout_msec;
+   write_concern->is_default = false;
+   write_concern->frozen = false;
 }
 
 
@@ -263,13 +248,12 @@ mongoc_write_concern_set_wmajority (mongoc_write_concern_t *write_concern,
 {
    BSON_ASSERT (write_concern);
 
-   if (!_mongoc_write_concern_warn_frozen (write_concern)) {
-      write_concern->w = MONGOC_WRITE_CONCERN_W_MAJORITY;
-      write_concern->is_default = false;
+   write_concern->w = MONGOC_WRITE_CONCERN_W_MAJORITY;
+   write_concern->is_default = false;
+   write_concern->frozen = false;
 
-      if (wtimeout_msec >= 0) {
-         write_concern->wtimeout = wtimeout_msec;
-      }
+   if (wtimeout_msec >= 0) {
+      write_concern->wtimeout = wtimeout_msec;
    }
 }
 
@@ -293,12 +277,11 @@ mongoc_write_concern_set_wtag (mongoc_write_concern_t *write_concern,
 {
    BSON_ASSERT (write_concern);
 
-   if (!_mongoc_write_concern_warn_frozen (write_concern)) {
-      bson_free (write_concern->wtag);
-      write_concern->wtag = bson_strdup (wtag);
-      write_concern->w = MONGOC_WRITE_CONCERN_W_TAG;
-      write_concern->is_default = false;
-   }
+   bson_free (write_concern->wtag);
+   write_concern->wtag = bson_strdup (wtag);
+   write_concern->w = MONGOC_WRITE_CONCERN_W_TAG;
+   write_concern->is_default = false;
+   write_concern->frozen = false;
 }
 
 /**
@@ -307,13 +290,8 @@ mongoc_write_concern_set_wtag (mongoc_write_concern_t *write_concern,
  *
  * This is an internal function.
  *
- * Freeze the write concern if necessary and retrieve the encoded bson_t
- * representing the write concern.
- *
- * You may not modify the write concern further after calling this function.
- *
- * Returns: A bson_t that should not be modified or freed as it is owned by
- *    the mongoc_write_concern_t instance.
+ * Returns: A bson_t representing the write concern, which is owned by the
+ *    mongoc_write_concern_t instance and should not be modified or freed.
  */
 const bson_t *
 _mongoc_write_concern_get_bson (mongoc_write_concern_t *write_concern)
@@ -326,41 +304,14 @@ _mongoc_write_concern_get_bson (mongoc_write_concern_t *write_concern)
 }
 
 /**
- * mongoc_write_concern_get_gle:
+ * mongoc_write_concern_is_default:
  * @write_concern: A mongoc_write_concern_t.
  *
- * This is an internal function.
- *
- * Freeze the write concern if necessary and retrieve the encoded bson_t
- * representing the write concern as a get last error command.
- *
- * You may not modify the write concern further after calling this function.
- *
- * Returns: A bson_t that should not be modified or freed as it is owned by
- *    the mongoc_write_concern_t instance.
- */
-const bson_t *
-_mongoc_write_concern_get_gle (mongoc_write_concern_t *write_concern)
-{
-   if (!write_concern->frozen) {
-      _mongoc_write_concern_freeze (write_concern);
-   }
-
-   return &write_concern->compiled_gle;
-}
-
-
-/**
- * _mongoc_write_concern_is_default:
- * @write_concern: A mongoc_write_concern_t.
- *
- * This is an internal function.
- *
  * Returns is_default, which is true when write_concern has not been modified.
  *
  */
 bool
-_mongoc_write_concern_is_default (const mongoc_write_concern_t *write_concern)
+mongoc_write_concern_is_default (const mongoc_write_concern_t *write_concern)
 {
    return !write_concern || write_concern->is_default;
 }
@@ -372,26 +323,21 @@ _mongoc_write_concern_is_default (const mongoc_write_concern_t *write_concern)
  *
  * This is an internal function.
  *
- * Freeze the write concern if necessary and encode it into a bson_ts which
- * represent the raw bson form and the get last error command form.
- *
- * You may not modify the write concern further after calling this function.
+ * Encodes the write concern into a bson_t, which may then be returned by
+ * mongoc_read_concern_get_bson().
  */
 static void
 _mongoc_write_concern_freeze (mongoc_write_concern_t *write_concern)
 {
    bson_t *compiled;
-   bson_t *compiled_gle;
 
    BSON_ASSERT (write_concern);
 
    compiled = &write_concern->compiled;
-   compiled_gle = &write_concern->compiled_gle;
 
    write_concern->frozen = true;
 
-   bson_init (compiled);
-   bson_init (compiled_gle);
+   bson_reinit (compiled);
 
    if (write_concern->w == MONGOC_WRITE_CONCERN_W_TAG) {
       BSON_ASSERT (write_concern->wtag);
@@ -415,9 +361,6 @@ _mongoc_write_concern_freeze (mongoc_write_concern_t *write_concern)
    if (write_concern->wtimeout) {
       bson_append_int32 (compiled, "wtimeout", 8, write_concern->wtimeout);
    }
-
-   BSON_APPEND_INT32 (compiled_gle, "getlasterror", 1);
-   bson_concat (compiled_gle, compiled);
 }
 
 
@@ -476,7 +419,7 @@ mongoc_write_concern_is_valid (const mongoc_write_concern_t *write_concern)
 }
 
 
-bool
+static bool
 _mongoc_write_concern_validate (const mongoc_write_concern_t *write_concern,
                                 bson_error_t *error)
 {
@@ -484,7 +427,7 @@ _mongoc_write_concern_validate (const mongoc_write_concern_t *write_concern,
       bson_set_error (error,
                       MONGOC_ERROR_COMMAND,
                       MONGOC_ERROR_COMMAND_INVALID_ARG,
-                      "Invalid mongoc_write_concern_t");
+                      "Invalid writeConcern");
       return false;
    }
    return true;
@@ -569,97 +512,69 @@ mongoc_write_concern_append (mongoc_write_concern_t *write_concern,
  *    with mongoc_write_concern_destroy().
  */
 mongoc_write_concern_t *
-_mongoc_write_concern_new_from_iter (bson_iter_t *iter)
+_mongoc_write_concern_new_from_iter (bson_iter_t *iter, bson_error_t *error)
 {
    bson_iter_t inner;
    mongoc_write_concern_t *write_concern;
 
    BSON_ASSERT (iter);
-   write_concern =
-      (mongoc_write_concern_t *) bson_malloc0 (sizeof *write_concern);
-   write_concern->w = MONGOC_WRITE_CONCERN_W_DEFAULT;
-   write_concern->fsync_ = MONGOC_WRITE_CONCERN_FSYNC_DEFAULT;
-   write_concern->journal = MONGOC_WRITE_CONCERN_JOURNAL_DEFAULT;
+
+   write_concern = mongoc_write_concern_new ();
 
    BSON_ASSERT (bson_iter_recurse (iter, &inner));
    while (bson_iter_next (&inner)) {
       if (BSON_ITER_IS_KEY (&inner, "w")) {
          if (BSON_ITER_HOLDS_INT32 (&inner)) {
-            write_concern->w = bson_iter_int32 (&inner);
+            mongoc_write_concern_set_w (write_concern,
+                                        bson_iter_int32 (&inner));
          } else if (BSON_ITER_HOLDS_UTF8 (&inner)) {
             if (!strcmp (bson_iter_utf8 (&inner, NULL), "majority")) {
-               write_concern->w = MONGOC_WRITE_CONCERN_W_MAJORITY;
+               /* mongoc_write_concern_set_wmajority() only assigns wtimeout if
+                * it is >= 0. Since we set wtimeout below, pass -1 here. */
+               mongoc_write_concern_set_wmajority (write_concern, -1);
             } else {
-               write_concern->w = MONGOC_WRITE_CONCERN_W_TAG;
-               write_concern->wtag = bson_iter_dup_utf8 (&inner, NULL);
+               mongoc_write_concern_set_wtag (write_concern,
+                                              bson_iter_utf8 (&inner, NULL));
             }
+         } else {
+            /* wrong type for "w" */
+            goto fail;
          }
-      } else if (BSON_ITER_IS_KEY (&inner, "fsync") &&
-                 BSON_ITER_HOLDS_BOOL (&inner)) {
-         write_concern->fsync_ = bson_iter_bool (&inner);
-      } else if (BSON_ITER_IS_KEY (&inner, "j") &&
-                 BSON_ITER_HOLDS_BOOL (&inner)) {
-         write_concern->journal = bson_iter_bool (&inner);
-      } else if (BSON_ITER_IS_KEY (&inner, "wtimeout") &&
-                 BSON_ITER_HOLDS_INT32 (&inner)) {
-         write_concern->wtimeout = bson_iter_bool (&inner);
+      } else if (BSON_ITER_IS_KEY (&inner, "fsync")) {
+         if (!BSON_ITER_HOLDS_BOOL (&inner)) {
+            goto fail;
+         }
+         BEGIN_IGNORE_DEPRECATIONS;
+         mongoc_write_concern_set_fsync (write_concern,
+                                         bson_iter_bool (&inner));
+         END_IGNORE_DEPRECATIONS;
+      } else if (BSON_ITER_IS_KEY (&inner, "j")) {
+         if (!BSON_ITER_HOLDS_BOOL (&inner)) {
+            goto fail;
+         }
+         mongoc_write_concern_set_journal (write_concern,
+                                           bson_iter_bool (&inner));
+      } else if (BSON_ITER_IS_KEY (&inner, "wtimeout")) {
+         if (!BSON_ITER_HOLDS_INT32 (&inner) || bson_iter_int32 (&inner) < 0) {
+            goto fail;
+         }
+         mongoc_write_concern_set_wtimeout (write_concern,
+                                            bson_iter_int32 (&inner));
       }
    }
 
+   if (!_mongoc_write_concern_validate (write_concern, error)) {
+      mongoc_write_concern_destroy (write_concern);
+      return NULL;
+   }
+
    return write_concern;
-}
 
-/**
- * _mongoc_write_concern_iter_is_valid:
- * @iter: (in): A bson_iter_t positioned on a "writeConcern" BSON document.
- *
- * Checks to see if @write_concern is valid and does not contain conflicting
- * options.
- *
- * Returns: true if the write concern is valid; otherwise false.
- */
-bool
-_mongoc_write_concern_iter_is_valid (bson_iter_t *iter)
-{
-   bson_iter_t inner;
-   bool has_fsync = false;
-   bool w0 = false;
-   bool j = false;
-
-   BSON_ASSERT (iter);
-   BSON_ASSERT (bson_iter_recurse (iter, &inner));
-   while (bson_iter_next (&inner)) {
-      if (BSON_ITER_IS_KEY (&inner, "fsync")) {
-         if (!BSON_ITER_HOLDS_BOOL (&inner)) {
-            return false;
-         }
-         has_fsync = bson_iter_bool (&inner);
-      } else if (BSON_ITER_IS_KEY (&inner, "w")) {
-         if (BSON_ITER_HOLDS_INT32 (&inner)) {
-            if (bson_iter_int32 (&inner) ==
-                   MONGOC_WRITE_CONCERN_W_UNACKNOWLEDGED ||
-                bson_iter_int32 (&inner) ==
-                   MONGOC_WRITE_CONCERN_W_ERRORS_IGNORED) {
-               w0 = true;
-            }
-         } else if (!(BSON_ITER_HOLDS_UTF8 (&inner))) {
-            return false;
-         }
-      } else if (BSON_ITER_IS_KEY (&inner, "j")) {
-         if (!BSON_ITER_HOLDS_BOOL (&inner)) {
-            return false;
-         }
-         j = bson_iter_bool (&inner);
-      } else if (BSON_ITER_IS_KEY (&inner, "wtimeout")) {
-         if (!BSON_ITER_HOLDS_INT32 (&inner) || bson_iter_int32 (&inner) < 0) {
-            return false;
-         }
-      }
-   }
-
-   if ((has_fsync || j) && w0) {
-      return false;
-   }
-
-   return true;
+fail:
+   bson_set_error (error,
+                   MONGOC_ERROR_COMMAND,
+                   MONGOC_ERROR_COMMAND_INVALID_ARG,
+                   "Invalid writeConcern");
+   mongoc_write_concern_destroy (write_concern);
+   return NULL;
 }
